/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann   *
 *   dirk.hartmann@sitter-net.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "gpgaccess.h"
#include "config.h"
#include <fstream>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "smartbeatlib.h"

#define BUFFERSIZE 2048
/* NOTE gpg answer longer than 1024 (Buffersize) will be ignored */

GPGAccess::GPGAccess(const std::string &gpg_command, const std::string &gpg_homedir, const std::string &gpg_key)
{
	init(gpg_command, gpg_homedir, gpg_key);
}

GPGAccess::GPGAccess()
{
	// just do nothing
}

void GPGAccess::init()
{
	/* when this is the first instance */
	int success;
	success = 0;

	if (gpg_homedir[gpg_homedir.length()-1] != '/')
		gpg_homedir += "/";
}

void GPGAccess::init(const std::string & gpg_command, const std::string & gpg_homedir, const std::string & gpg_key)
{
	GPGAccess::gpg_homedir = gpg_homedir;
	GPGAccess::gpg_command = gpg_command;
	GPGAccess::gpg_key = gpg_key;
	init();
}


#ifdef WORKAROUND
std::string GPGAccess::encode_message(const std::string &msg) const
{
	FILE* pipe;
	int bytes;
	int total_bytes;
	std::string ret, command;
	ret = "";

	/* create buffer space */
	char filebuffer[BUFFERSIZE];

	command = "echo $'" + msg + "'|" + gpg_command + " -e -a -q --no-tty --batch --no-permission-warning --no-secmem-warning --homedir " + gpg_homedir + " -r " + gpg_key + " 2> /dev/null";
	/*open the stream via popen*/
	pipe = popen (command.c_str(), "r");
	
	/*read from the stream*/
	bytes = fread (filebuffer, 1, BUFFERSIZE, pipe);
	filebuffer[bytes] = '\0'; //terminate the string
	ret = filebuffer;
	/*now close the pipe and clean up the child*/
	pclose(pipe);
	return ret;
}

#else

std::string GPGAccess::encode_message(const std::string &msg) const
{	
	int in[2];
	int out[2];
	int bytes_read;
	std::string ret;
// 	std::cout << "IN: " << STDIN_FILENO << " OUT: " << STDOUT_FILENO << std::endl;

	pid_t pid;
	ret = "";
	/* Create a pipe.  File descriptors for the two ends of the pipe are
	placed in fds.  */
	socketpair(AF_UNIX, SOCK_STREAM, 0, in);
	fcntl(in[0], F_SETFD, FD_CLOEXEC);
	fcntl(in[1], F_SETFD, FD_CLOEXEC);

	socketpair(AF_UNIX, SOCK_STREAM, 0, out);
	fcntl(out[0], F_SETFD, FD_CLOEXEC);
	fcntl(out[1], F_SETFD, FD_CLOEXEC);
	/* Fork a child process.  */
	pid = fork ();
	if (pid == -1)
		return ret;
	else if (pid ==  0)
	{
		/* This is the child process.  Close our copy of the write end of
		the file descriptor.  */
		close (in[1]);
		close (out[0]);
		/* Connect the read end of the pipe to standard input.  */
		dup2 (in[0], STDIN_FILENO);
		dup2 (out[1], STDOUT_FILENO);
		/* Replace the child process with the "gpg" program.  */
		// sleep(5);
		execl(gpg_command.c_str(), gpg_command.c_str(), "-e", "-a", "-q", "--no-tty", "--batch" ,"--no-permission-warning",  "--no-secmem-warning",  "--homedir",  gpg_homedir.c_str(), "-r", gpg_key.c_str(), NULL);
		perror("gpg errors");
		_exit(0);
	}
	else 
	{
		/* This is the parent process.  */
		FILE* stream;
		FILE* stream2;
		/* Convert the write file descriptor to a FILE object, and write
		to it.  */
		stream = fdopen (in[1], "w");
		fprintf (stream, msg.c_str());
		fflush (stream);
		// errno = 0;
		// int tmp = write(in[0], msg.c_str(), msg.length()+1);
		// std::cout << tmp << " " << errno << std::endl;
		/* Close our copy of the read end of the file descriptor.  */
		close (in[0]);
		close (out[1]); //close write end
		close (in[1]);
		stream2 = fdopen (out[0], "r");
		char filebuffer[BUFFERSIZE];

		bytes_read = fread (filebuffer, 1, BUFFERSIZE, stream2);
		filebuffer[bytes_read] = '\0'; //terminate the string
		close(out[0]);
		waitpid(pid, NULL, 0);
		ret = filebuffer;
		fclose(stream);
		fclose(stream2);
		return (ret);
	}
}

#endif


GPGAccess::~GPGAccess()
{
}
