/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "smartbeatconfig.h"
#include "smartbeatlogger.h"
#include "config.h"

#include <iostream>
#include <fstream>
#include <string.h>

SmartBeatConfig::SmartBeatConfig()
{
	setStdConfigParameters();
}


SmartBeatConfig::~SmartBeatConfig()
{
	delete SBLogger;
}

void SmartBeatConfig::setStdConfigParameters()
{
	port = 5000;
	pinganswer = "heartbeat"; //answer from the client
	gpg_home = "/var/lib/smartbeat/.gnupg"; //optional parameters
	gpg = "/usr/bin/gpg"; //path to gpg or gpg command
	logdir = "/var/log/smartbeat/";
	server_address = "sitter-net.de";
	get_ip_url = "http://www.sitter-net.de/cgi-bin/whatismyip.pl";
	retry_time = 60; // time till next connection retry
	timout_time = 100; // 100 secs timeout
	my_id = "0";
	my_pw = "baum";
	gpg_key = "hardware@sitter-net.de"; // key used for encryption
	config_file = SYSCONFDIR;
	config_file += "/smartbeatclient.conf";
	coloumn_separator = " ";
	row_separator = "|";
	sensor_command = "/etc/init.d/lm_sensors status";
	logMessages = false;
	
	SBLogger = new SmartBeatLogger(logdir, "smartbeatclient.log");
}

vector< string > SmartBeatConfig::splitString(const string & c1, int sep)
{
	bool cont = false;
	string tmp("");
	vector< string > ret;
	
	for (unsigned int i = 0; i < c1.length(); ++i) {
		cont = true;
		if (sep & Separated::Space)
			if( c1[i] == ' ') {
				cont = false;
			}
		if (sep & Separated::Tabbed)
			if (c1[i] == '\t') {
				cont = false;
			}
		if (sep & Separated::Line)
			if (c1[i] == '\n') {
				cont = false;
			}
		if (cont) {
			tmp += c1[i];
		} else {
			if (!tmp.empty())
				ret.push_back(tmp);
			tmp = "";
		}
	}
	if (!tmp.empty())
		ret.push_back(tmp);
	
	return ret;
}

string SmartBeatConfig::joinString(const vector< string > & strVector) const
{
	string joinedString;
	
	for (unsigned int i = 0; i < strVector.size(); ++i) {
		if (i == 0)
			joinedString = strVector[i];
		else
			joinedString += " ";
			joinedString += strVector[i];
	}
	return joinedString;
}

bool SmartBeatConfig::readConfigFromFile(const string & fileName)
{
	string line;
	vector<string> elems;
	
	if (!fileName.empty())
		config_file = fileName; // store config file
	
	std::ifstream file (config_file.c_str()); // open file
	
	if (!file.is_open()) // could not open file
		return false;
	
	while (!file.eof()) {
		std::getline(file, line);
		line = getUncommented(line);
		if (line.size() > 0) { // line is not empty
			elems = splitString(line, Separated::Space|Separated::Tabbed);
			setParam(elems);
		}
	}
	file.close();
	checkForEntryDBEntries();
	return true;
}

string SmartBeatConfig::getUncommented(const string & s)
{
	string tmp;
	int endPos; // for creating substrings
	
	endPos = s.find_first_of('#'); // search for first comment char
	if (endPos == 0) // whole line is a comment
		return "";
	
	if (endPos > 0) // found # comment in line
		tmp = s.substr(0, endPos);
	else // no comment in line take whole line
		tmp = s;
	
	endPos = tmp.find_first_of(';');
	if (endPos == 0) // whole line is a comment
		return "";
	
	if (endPos > 0) // found ; comment in line
		tmp = s.substr(0, endPos);
	return tmp;
	
}

void SmartBeatConfig::setParam(vector< string > & elems)
{
	if (elems.size() < 2) // no pair
		return;
	
	if ( strcasecmp( elems[0].c_str(), "PORT" ) == 0 ) port = atoi(elems[1].c_str());
	else if ( strcasecmp( elems[0].c_str(), "PINGANSWER" ) == 0 ) pinganswer = atoi(elems[1].c_str());
	else if ( strcasecmp( elems[0].c_str(), "GPG" ) == 0 ) gpg = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "RETRY_TIME" ) == 0 ) retry_time = atoi(elems[1].c_str());
	else if ( strcasecmp( elems[0].c_str(), "GPGHOME" ) == 0 ) gpg_home = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "GET_IP_URL" ) == 0 ) get_ip_url = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "SERVER_ADDRESS" ) == 0 ) server_address = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "TIMEOUT_TIME" ) == 0 ) timout_time = atoi(elems[1].c_str());
	else if ( strcasecmp( elems[0].c_str(), "ID" ) == 0 ) my_id = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "GPG_ENCRYPTION_KEY" ) == 0 ) gpg_key = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "PASSWORD" ) == 0 ) my_pw = elems[1];
	else if ( strcasecmp( elems[0].c_str(), "SENSORS" ) == 0 ) sensor_command = getQuoted(joinString(elems));
	else if ( strcasecmp( elems[0].c_str(), "ROW_SEPARATOR" ) == 0 ) row_separator = getQuoted(joinString(elems));
	else if ( strcasecmp( elems[0].c_str(), "COLOUMN_SEPARATOR" ) == 0 ) coloumn_separator = getQuoted(joinString(elems));
	else if ( strcasecmp( elems[0].c_str(), "LOG_MESSAGES" ) == 0 ) {
		if (strcasecmp (elems[1].c_str(), "YES" ) == 0)
			logMessages = true;
		else
			logMessages = false;
	} else if ( strcasecmp( elems[0].c_str(), "LOGDIR" ) == 0 ){
		/* check for trailing '/' */
		if (elems[1][elems[1].length()-1] != '/') {
			logdir = elems[1];
			logdir += "/";
			SBLogger->set_complete_log_path(logdir + "smartbeatclient.log");
		} else {
			logdir = elems[1];
			SBLogger->set_complete_log_path(logdir + "smartbeatclient.log");
		}
	} else if ( strcasecmp( elems[0].c_str(), "ADD_MONITORED_DISK" ) == 0 ) {
		if (elems.size() > 2 && monitored_disks.size() < MaxMapSize) {
			monitored_disks.insert(make_pair(elems[1], elems[2]));
			SBLogger->write_to_log("INFO: adding "+ elems[2] +" with name "+ elems[1] +" to monitored disks");
		} else {
			SBLogger->write_to_log("INFO: adding disk failed: too many items or missing option");
		}
	} else if ( strcasecmp( elems[0].c_str(), "ADD_SENSOR" ) == 0 ) {
		if (elems.size() > 2 && sensors.size() < MaxMapSize) {
			string sensorName = getQuoted(joinString(elems));
			if (!sensorName.empty()) {
				sensors.insert(make_pair(elems[1], sensorName));
				SBLogger->write_to_log("INFO: added sensor \""+ sensorName + "\" as "+ elems[1]);
			}
		} else {
			SBLogger->write_to_log("INFO: adding sensor failed: too many items or missing option");
		}
	} else if ( strcasecmp( elems[0].c_str(), "ADD_CUSTOM_COMMAND" ) == 0 ) {
		if (elems.size() > 2 && custom_commands.size() < MaxMapSize) {
			string command = getQuoted(joinString(elems));
			custom_commands.insert(make_pair(elems[1], command));
			SBLogger->write_to_log("INFO: adding custom command: identifier "+ elems[1] +"; command \""+ command +"\"");
		} else {
			SBLogger->write_to_log("INFO: adding custom command failed: too many items or missing option");
		}
	} else if ( strcasecmp( elems[0].c_str(), "ADD_DATABASE" ) == 0 ) {
		// check for already existing entry
		map<string, DatabaseSettings>::iterator databaseIter = mysqlDatabaseSettings.find(elems[1]);
		if (databaseIter != mysqlDatabaseSettings.end()) {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" database already exists in config");
		} else if (elems.size() > 4 && mysqlDatabaseSettings.size() < MaxMapSize) {
			DatabaseSettings settings; // create and store settings
			settings.dbHost = elems[2];
			settings.dbUser = elems[3];
			settings.dbPass = elems[4];
			
			// insert database into config
			mysqlDatabaseSettings.insert(make_pair(elems[1], settings));
			SBLogger->write_to_log("INFO: added database "+ elems[1]);
		} else {
			SBLogger->write_to_log("WARNING: invalid config for "+ elems[1] +" or too many items");
		}
	} else if ( strcasecmp( elems[0].c_str(), "ADD_MYSQL_QUERY" ) == 0 ) {
		// check for existing database
		map<string, DatabaseSettings>::iterator databaseIter = mysqlDatabaseSettings.find(elems[1]);
		
		if (databaseIter == mysqlDatabaseSettings.end()) {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" database does not exists in config."
				"you have to add the database before");
		} else if (elems.size() > 3 && databaseIter->second.dbQueries.size() < MaxMapSize) {
			string mysqlQuery = getQuoted(joinString(elems)); // get quoted part
			
			if (!mysqlQuery.empty()) {
				// std::cout << elems[2] << std::endl;
				databaseIter->second.dbQueries.insert(make_pair(elems[2], mysqlQuery));
				SBLogger->write_to_log("INFO: "+ elems[1] +" added query \""+ mysqlQuery +"\"");
			} else {
				SBLogger->write_to_log("WARNING: "+ elems[1] +" invalid query not added");
			}
		} else {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" invalid query not added or too many items");
		}
	}
}

void SmartBeatConfig::checkForEntryDBEntries()
{
	map<string, DatabaseSettings>::iterator databaseIter;
	vector<string> entriesToRemove;
	
	// collects unused databases
	for (databaseIter = mysqlDatabaseSettings.begin(); databaseIter != mysqlDatabaseSettings.end();
		databaseIter++) {
		if (databaseIter->second.dbQueries.empty()) {
			entriesToRemove.push_back(databaseIter->first);
		}
	}
	
	// now remove them from map
	for (unsigned int i = 0; i < entriesToRemove.size(); ++i) {
		databaseIter = mysqlDatabaseSettings.find(entriesToRemove[i]);
		mysqlDatabaseSettings.erase(databaseIter);
		SBLogger->write_to_log("INFO: removed unused database "+ entriesToRemove[i]);
	}
}

string SmartBeatConfig::getQuoted(const string & s) const
{
	int startPos = s.find_first_of('\"') + 1; // dont use the quote as starting value
	int endPos = s.find_first_of('\"', startPos + 1);
	if (startPos > 0 && endPos > startPos)
		return s.substr(startPos, endPos - startPos);
	return "";
}

bool SmartBeatConfig::reloadConfig()
{
	bool ret;
	monitored_disks.clear();
	SBLogger->write_to_log("Reloading Config...");
	ret = readConfigFromFile();
	SBLogger->open();
	if (ret)
		SBLogger->write_to_log("... Success");
	else
		SBLogger->write_to_log("... Failed");
	return ret;
}



void SmartBeatConfig::setSmartBeatLogger ( SmartBeatLogger* newLogger )
{
	delete SBLogger;
	SBLogger = newLogger;
}

void SmartBeatConfig::setServerPort ( int newPort )
{
	port = newPort;
}

void SmartBeatConfig::setClientPinganswer ( const string& newAnswer )
{
	pinganswer = newAnswer;
}

void SmartBeatConfig::setGpgHomeDir ( const string& newHome )
{
	gpg_home = newHome;
}


void SmartBeatConfig::setReconnectTime ( int newTime )
{
	retry_time = newTime;
}

void SmartBeatConfig::setGetMyIpUrl ( const string& newGetIpUrl )
{
	get_ip_url = newGetIpUrl;
}

void SmartBeatConfig::setSmartBeatServerAddress ( const string& newServerAddress )
{
	server_address = newServerAddress;
}

void SmartBeatConfig::setSocketTimout ( int newTimeout )
{
	timout_time = newTimeout;
}

void SmartBeatConfig::setSmartBeatId ( const string& newId )
{
	my_id = newId;
}

void SmartBeatConfig::setSmartBeatPassword ( const string& newPw )
{
	my_pw = newPw;
}

void SmartBeatConfig::setGpgKeyName ( const string& newKey )
{
	gpg_key = newKey;
}

void SmartBeatConfig::setLogDir ( const string& newLogDir )
{
	logdir = newLogDir;
}

void SmartBeatConfig::setConfigFileName ( const string& newConfigFile )
{
	config_file = newConfigFile;
}

void SmartBeatConfig::setLogSmartBeatMessages ( bool newLogState )
{
	logMessages = newLogState;
}


void SmartBeatConfig::setGpgCommand ( const string& newCommand )
{
	gpg = newCommand;
}

void SmartBeatConfig::setRowSeparator(const string & newSeparator)
{
	row_separator = newSeparator;
}

void SmartBeatConfig::setColoumnSeparator(const string & newSeparator)
{
	coloumn_separator = newSeparator;
}

void SmartBeatConfig::setSensorCommand(const string & command)
{
	sensor_command = command;
}
