/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SMARTBEATCONFIG_H
#define SMARTBEATCONFIG_H

#include <string>
#include <vector>
#include <map>

using ::std::string;
using ::std::vector;
using ::std::map;

class SmartBeatLogger;

enum DBMode {UNUSED = 0x0000001, ZM_MODE = 0x0000002};

namespace Separated {
	enum {Line = 0x1, Tabbed = 0x2, Space = 0x4};
	// seperate by line (\n) tab (\t) or space ( )
}

struct DatabaseSettings {
	// the needed things for a user defined mysql query
	string dbHost;
	string dbUser;
	string dbPass;
	map <string, string> dbQueries; // to perform multiple queries on a single
					// database <query identifer, query>
};

class SmartBeatConfig {
	public:
		SmartBeatConfig();

		~SmartBeatConfig();

		void setStdConfigParameters(); // sets std config parameters
		bool readConfigFromFile ( const string &fileName = "" ); // reads the config parameters from the given file
		bool reloadConfig();
		
		vector<string> splitString ( const string &c1, int sep ); // spkits a string
		string joinString (const vector<string> &strVector) const;

		void setSmartBeatLogger ( SmartBeatLogger* newLogger );
		SmartBeatLogger* logger() const {return SBLogger;}

		void setServerPort ( int newPort );
		int serverPort() const {return port;}

		void setClientPinganswer ( const string& newAnswer );
		string clientPinganswer() const {return pinganswer;}

		void setGpgHomeDir ( const string& newHome );
		string gpgHomeDir() const {return gpg_home;}

		void setReconnectTime ( int newTime );
		int reconnectTime() const {return retry_time;}

		void setGetMyIpUrl ( const string& newGetIpUrl );
		string getMyIpUrl() const {return get_ip_url;}

		void setSmartBeatServerAddress ( const string& newServerAddress );
		string smartBeatServerAddress() const {return server_address;}

		void setSocketTimout ( int newTimeout );
		int socketTimout() const {return timout_time;}

		void setSmartBeatId ( const string& newId );
		string smartBeatId() const {return my_id;}

		void setSmartBeatPassword ( const string& newPw );
		string smartBeatPassword() const {return my_pw;}

		void setGpgKeyName ( const string& newKey );
		string gpgKeyName() const {return gpg_key;}

		void setLogDir ( const string& newLogDir );
		string logDir() const {return logdir;}

		void setConfigFileName ( const string& newConfigFile );
		string configFileName() const {return config_file;}

		void setLogSmartBeatMessages ( bool newLogState );
		bool logSmartBeatMessages() const {return logMessages;}

		void setGpgCommand ( const string& newCommand );
		string gpgCommand() const {return gpg;}
		
		void setRowSeparator( const string &newSeparator );
		string rowSeparator() const {return row_separator;}
		
		void setColoumnSeparator( const string &newSeparator );
		string coloumnSeparator() const {return coloumn_separator;}
		
		void setSensorCommand( const string &command );
		string sensorCommand() const {return sensor_command;}

		const map< string, string > * monitoredDisks() const {return &monitored_disks;}
		const map< string, DatabaseSettings > * databaseSettings() const {return &mysqlDatabaseSettings;}
		const map< string, string > * customCommands() const {return &custom_commands;}
		const map<string, string> * sensorNames() const {return &sensors;}

	protected:
		string getUncommented(const string &s); // extracts uncommented part
		string getQuoted(const string &s) const; // gets the quoted query from a string
		void checkForEntryDBEntries(); // checks for databases where no query is denfined (so it's unused)

		void setParam ( vector<string> &elems ); // sets the config values

		// protected vars
		map<string, string> monitored_disks; // <disk identifier, path to disk>
		// databases on which queries are defined
		map<string, DatabaseSettings> mysqlDatabaseSettings; // <dbName, settings/queries>
		map<string, string> sensors; // all sensor identifiers and names <identifier, name>
		map<string, string> custom_commands; // <command identifier, command>
		
		SmartBeatLogger* SBLogger; // pointer to the logger
		int port;
		string pinganswer; //answer from the client
		string gpg_home; //optional gpg homedir
		string gpg; //path to gpg or gpg command
		int retry_time; // time for till next connection retry
		string get_ip_url; // url for retrieving the host ip
		string server_address; // smartbeat server address
		int timout_time; // timeout time in secs for non blocking socket
		string my_id; // smartbeat id for this client
		string my_pw; // password for smartbeat id
		string gpg_key;
		string logdir;
		string config_file;
		string row_separator; // seperator for the result rows in a mysql query
		string coloumn_separator; // seperator for the result coloumns in a mysql query
		string sensor_command; // sensors command
		bool logMessages;
};

#endif
