/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann   *
 *   dirk.hartmann@sitter-net.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "smartbeatlogger.h"
#include "smartbeatlib.h"

SmartBeatLogger::SmartBeatLogger() : 
	logfile("smartbeat_client.log"), logdir("/var/log/smartbeat")
{
	pthread_mutex_init (&logfile_mutex, NULL); // init mutex
	generate_complete_logpath();
}

SmartBeatLogger::SmartBeatLogger( const std::string &path, const std::string &file )
{
	pthread_mutex_init (&logfile_mutex, NULL); // init mutex
	logdir = path;
	logfile = file;
	generate_complete_logpath();
}

SmartBeatLogger::SmartBeatLogger(const std::string &completepath)
{
	pthread_mutex_init (&logfile_mutex, NULL); // init mutex
	complete_log_path = completepath;
}

void SmartBeatLogger::generate_complete_logpath()
{
	if (logdir[logdir.length()-1] != '/') // trailing /
		logdir += "/";
	complete_log_path = logdir + logfile;
}

void SmartBeatLogger::open( )
{	
	if (!file.is_open()) { // file is not open
		file.open(complete_log_path.c_str(), std::ios::app); // open in append mode
	}
	if (!file.is_open())
		printToStderr("ERROR: Could not open logfile. using STDERR instead\n");
}

void SmartBeatLogger::set_logdir(const std::string &new_logdir)
{
	bool reopen; // reopen file after change
	
	pthread_mutex_lock (&logfile_mutex); //threadsafe
	
	reopen = false;
	if (file.is_open()) {
		file.close();
		reopen = true;
	}
	logdir = new_logdir;
	generate_complete_logpath();
	
	if (reopen)
		open();
	pthread_mutex_unlock (&logfile_mutex); //threadsafe
}

void SmartBeatLogger::set_logfile( const std::string &new_logfile )
{
	bool reopen; // reopen file after change
	
	pthread_mutex_lock (&logfile_mutex); //threadsafe
	
	reopen = false;
	if (file.is_open()) {
		file.close();
		reopen = true;
	}
	logfile = new_logfile;
	generate_complete_logpath();
	
	if (reopen)
		open();
	pthread_mutex_unlock (&logfile_mutex); //threadsafe
}

void SmartBeatLogger::set_complete_log_path( const std::string &new_complete_log_path)
{
	bool reopen; // reopen file after change
	
	pthread_mutex_lock (&logfile_mutex); //threadsafe
	
	reopen = false;
	if (file.is_open()) {
		file.close();
		reopen = true;
	}
	complete_log_path = new_complete_log_path;
	
	if (reopen)
		open();
	pthread_mutex_unlock (&logfile_mutex); //threadsafe
}

void SmartBeatLogger::printToStderr(const std::string & msg)
{
	fprintf( stderr, msg.c_str() );
}

void SmartBeatLogger::write_to_log(const std::string &msg, const std::string &in_extra_bracket)
{
	pthread_mutex_lock (&logfile_mutex); //threadsafe
	std::string logfile, day, month, hour, min, year, sec, writemsg;
	time_t rawtime;
	struct tm * timeinfo;
	int int_sec, int_min; //for length checking
	
	time ( &rawtime );
	timeinfo = localtime ( &rawtime ); //get localtime
	day = int2str(timeinfo->tm_mday);
	month = int2str(timeinfo->tm_mon+1);
	hour = int2str(timeinfo->tm_hour);
	int_min = timeinfo->tm_min;
	year = int2str(timeinfo->tm_year + 1900);
	int_sec = timeinfo->tm_sec;

	sec = int2str(int_sec);
	min = int2str(int_min);
	/*einstellige uhrzeiten*/
	if (int_sec < 10)
	{
		sec = "0" + sec;
	}
	if (int_min < 10)
	{
		min = "0" + min;
	}
	
	if (in_extra_bracket == "")
	{
		writemsg = "[" + day + "." + month + "." + year + " " + hour + ":" + min + ":" + sec + "]" + msg + "\n";//set upt the format
	}
	else
	{
		writemsg = "[" + day + "." + month + "." + year + " " + hour + ":" + min + ":" + sec + "]" + "[" + in_extra_bracket + "]" + msg + "\n";//set upt the format
	}
	//std::cout << writemsg;
	
	if (!file.is_open())
		open();
	if (file.is_open()) {
		file << writemsg;
		file.flush();
	}
	else
		printToStderr(writemsg);
		
	pthread_mutex_unlock (&logfile_mutex); //threadsafe
// 	
// 	{
// 		fwrite(writemsg.c_str(), writemsg.length(),1 , file);
// 		fclose(file);
// 	}
// 	else
// 	{
// 		std::cout << "WARNING: could not write to logfile" << "\n";
// 		std::cout << "check permissions or settings in conf file" << "\n\n";
// 	}
// 	pthread_mutex_unlock (&logfile_mutex); //threadsafe
}

SmartBeatLogger::~SmartBeatLogger()
{
	file.close();
}


